// NEOPIXEL BEST PRACTICES for most reliable operation:
// - Add 1000 uF CAPACITOR between NeoPixel strip's + and - connections.
// - MINIMIZE WIRING LENGTH between microcontroller board and first pixel.
// - NeoPixel strip's DATA-IN should pass through a 300-500 OHM RESISTOR.
// - AVOID connecting NeoPixels on a LIVE CIRCUIT. If you must, ALWAYS
//   connect GROUND (-) first, then +, then data.
// - When using a 3.3V microcontroller with a 5V-powered NeoPixel strip,
//   a LOGIC-LEVEL CONVERTER on the data line is STRONGLY RECOMMENDED.
// (Skipping these may work OK on your workbench but can fail in the field)
//SOURCE FOR THE ABOVE HEADER ON NEOPIXEL BEST PRACTICES:
//  Arduino. (2021). Examples. [Arduino software]. Retrieved from https://www.arduino.cc/en/Tutorial/BuiltInExamples

/*
LED Control with Music Input
Manav Shah, Dannalee Mata, Evan Walker, Scott Rimkus
April 19, 2023
*/

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
 #include <avr/power.h> 
#endif

// Which pin on the Arduino is connected to the NeoPixels
#define LED_PIN    6

// number of leds
#define LED_COUNT 120

// ADC pin
#define ADC_PIN A0

// max and min signal levels
#define SIGNAL_MAX 25
#define SIGNAL_MIN 0

// max and min brightness
#define BRIGHTNESS_MAX 255
#define BRIGHTNESS_MIN 0

// colors
#define COLOR_RED   0xFF0000
#define COLOR_GREEN 0x00FF00
#define COLOR_BLUE  0x0000FF
#define COLOR_WHITE 0x

// NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);


// setup--------------------------
void setup() {
  // ADC config
  ADCSRA |= (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
  ADMUX |= (1 << REFS0);

  // LED strip intialize
  strip.begin();
  strip.show();
}

void loop() {
  // conversion
  ADCSRA |= (1 << ADSC);
    
  // wait for the conversion to finish
  while (ADCSRA & (1 << ADSC));
    
  // save signal level to ADC
  uint16_t signal = ADC;
    
  // signal level -> brightness
  uint8_t brightness = map(signal, SIGNAL_MIN, SIGNAL_MAX, BRIGHTNESS_MIN, BRIGHTNESS_MAX);
  strip.setBrightness(brightness);

  // signal level -> color
  uint32_t color;
  if (signal < SIGNAL_MAX / 3) { //signal is less than one-third of the max value
    color = COLOR_RED;
  } else if (signal < 2 * SIGNAL_MAX / 3) { //signal is between one-third and two-thirds of the max value
    color = COLOR_GREEN;
  } else {
    color = COLOR_BLUE; //signal is greater than two-thirds of the max value
  }
    
  for (int i = 0; i < LED_COUNT; i++) {
    strip.setPixelColor(i, color);
  }
  strip.show();
}
